#include "framerate.hh"
#include <limits>

uint32_t framerate_meter::tick(uint32_t now) {
    // remove frame timestamps from beginning that are too old
    uint32_t threshold = now >= window_ ? now - window_ : 0;
    while (start_ != end_ && ts_[start_] < threshold)
        start_ = (start_ + 1) % ts_cap_;

    // try to add new frame timestamp
    if ((end_ + 1) % ts_cap_ == start_) {
        // overflow. grow the buffer
        uint32_t* new_ts = new uint32_t[ts_cap_ * 2];
        int ni = 0;
        for (int i = start_; i != end_; i = (i + 1) % ts_cap_)
            new_ts[ni++] = ts_[i];

        delete[] ts_;
        ts_      = new_ts;
        ts_cap_ *= 2;
        start_   = 0;
        end_     = ni;
    }

    // now there's definitely room
    ts_[end_++] = now;
    end_ %= ts_cap_;

    // result is current time
    return now;
}

float framerate_meter::framerate() {
    if (end_ == start_)
        return std::numeric_limits<float>::quiet_NaN();

    int end = end_;
    if (end < start_)
        end += ts_cap_;

    int last = end_ - 1;
    if (last < 0)
        last += ts_cap_;

    return 1000.f * (end - start_) / float(ts_[last] - ts_[start_]);
}
