#ifndef _MESH_HH_
#define _MESH_HH_

#include "px/px.hh"

struct mesh {
private:
    struct ind3 {
        uint16_t vi, vti, vni;
        bool operator==(const ind3& i3) const {
            return vi == i3.vi && vti == i3.vti && vni == i3.vni;
        }
    };

    struct ind3_hash {
        std::hash<uint16_t> hu16;
        size_t operator()(const ind3& i3) const { return hu16(i3.vi); }
    };

    struct tri_info { ind3 inds[3]; };

    struct obj_info {
        std::vector<px::vec3> v;
        std::vector<px::vec2> vt;
        std::vector<px::vec3> vn;
        std::vector<tri_info> f;
    };

public:
    px::vec3* verts;
    px::vec3* normals;
    px::vec2* texcoords;
    uint16_t* tris;
    int       nverts;
    int       ntris;
    char      error[256];

    mesh()
        : verts(NULL)
        , normals(NULL)
        , texcoords(NULL)
        , tris(NULL)
        , nverts(0)
        , ntris(0) {
        error[0] = '\0';
    }

    ~mesh() { unload(); }

    bool load(const char* filename);
    void unload();

private:
    bool read_obj(const char* filename, obj_info& obj);
    // use void* instead of FILE* to avoid header dependency on <cstdio>
    bool read_obj(const char* filename, void* fp, obj_info& obj);
    bool read_line(const char* filename, int line_no,
                   char* tok[], int ntok, obj_info& obj);
    bool read_vec3(px::vec3& out, const char* x, const char* y, const char* z);
    bool read_ind3(ind3& out, char* s);
    bool split_ws(char* line, char* tokens[], int& ntok, int max_ntok);
};

#endif // _MESH_HH_
