#ifndef _PX_QLINEAR_HH_
#define _PX_QLINEAR_HH_

// requires px/record.hh

namespace px {

template<typename U>
struct qlinear {
    float rw;
    U val;

    qlinear() { }
    qlinear(float rw, const U& u) : rw(rw), val(u) { }
    U extract() const { return (1.f / rw) * val; }
};

template<>
struct qlinear<empty_rec_t> {
    qlinear() { }
    qlinear(float rw, const empty_rec_t& empty) { }
    empty_rec_t extract() const { return empty_rec; }
};

template<typename U>
inline qlinear<U> operator-(const qlinear<U>& a) {
    return qlinear<U>(-a.rw, -a.val);
}

template<typename U>
inline qlinear<U> operator+(const qlinear<U>& a, const qlinear<U>& b) {
    return qlinear<U>(a.rw + b.rw, a.val + b.val);
}

template<typename U>
inline qlinear<U> operator-(const qlinear<U>& a, const qlinear<U>& b) {
    return qlinear<U>(a.rw - b.rw, a.val - b.val);
}

template<typename U>
inline qlinear<U> operator*(float a, const qlinear<U>& b) {
    return qlinear<U>(a * b.rw, a * b.val);
}

template<>
inline qlinear<empty_rec_t> operator+(const qlinear<empty_rec_t>& a,
                                      const qlinear<empty_rec_t>& b) {
    return qlinear<empty_rec_t>();
}

template<>
inline qlinear<empty_rec_t> operator-(const qlinear<empty_rec_t>& a,
                                      const qlinear<empty_rec_t>& b) {
    return qlinear<empty_rec_t>();
}

template<>
inline qlinear<empty_rec_t> operator*(float a, const qlinear<empty_rec_t>& b) {
    return qlinear<empty_rec_t>();
}

template<typename U>
inline qlinear<U> mkqlinear(float rw, const U& u) { return qlinear<U>(rw, u); }

}      // namespace px
#endif // _PX_QLINEAR_HH_
