#ifndef _PX_RECORD_SIMD_HH_
#define _PX_RECORD_SIMD_HH_

// requires px/simd.hh, px/record.hh

namespace px {

template<>
struct simd<empty_rec_t> {
    typedef empty_rec_t type;
    static type make(const empty_rec_t& a, const empty_rec_t& b,
                     const empty_rec_t& c, const empty_rec_t& d) {
        return empty_rec;
    }
    static type get(const type& q, int i) { return empty_rec; }
    static void set(type& q, int i, const empty_rec_t& mt) { }
    static type choose(
        const mask_4& m, const type& then, const type& els) {
        return empty_rec;
    }
};

template<typename L, typename V, typename R>
struct simd<record<L,V,R> > {
    typedef typename simd<V>::type simd_V;
    typedef typename simd<R>::type simd_R;
    typedef record<L, simd_V, simd_R> type;

    static type make(const record<L,V,R>& a, const record<L,V,R>& b,
                     const record<L,V,R>& c, const record<L,V,R>& d) {
        return record<L, simd_V, simd_R>(
            simd<V>::make(a.head_, b.head_, c.head_, d.head_),
            simd<R>::make(a.tail_, b.tail_, c.tail_, d.tail_));
    }

    static record<L,V,R> get(const type& q, int i) {
        return record<L,V,R>(
            simd<V>::get(q.head_, i),
            simd<R>::get(q.tail_, i));
    }

    static void set(type& q, int i, const record<L,V,R>& rec) {
        simd<V>::set(q.head_, i, rec.head_);
        simd<R>::set(q.tail_, i, rec.tail_);
    }

    static type choose(
        const mask_4& m, const type& then, const type& els) {
        typedef simd<V> sv;
        typedef simd<R> sr;
        return type(
            sv::choose(m, then.head_, els.head_),
            sr::choose(m, then.tail_, els.tail_));
    }
};

}      // namespace px
#endif // _PX_RECORD_SIMD_HH_
